@@ -23,35 +23,44 @@
 #--------------------------------------------------------------------------------
 #
 
-import os, errno, pwd, sys, math, sqlite3, time, calendar
+import os, errno, pwd, sys, math, sqlite3, time, calendar, evolution
 from PyQt4 import QtCore, QtGui
 from www2sms_gui import *
-from www2sms_providers import *
+import www2sms_providers
 
 USER = pwd.getpwuid(os.getuid())[0]
 CONFIG_DIR = "/home/%s/MyDocs/www2sms" %(USER)
 
-#List of provider we support in www2sms_providers.py
-list_providers = ['o2online']
+#Get the list of providers based on the classes implemented.
+list_providers = []
+providers = []
+for label in dir(www2sms_providers):
+    if label.startswith('provider_'):
+        provider_class = getattr(www2sms_providers, label)
+        provider = provider_class()
+	list_providers.append(provider.getName())
+	providers.append(provider)
 
 class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 	def __init__(self, win_parent = None):
-                #Init the base class
+		#Init the base class
 		QtGui.QMainWindow.__init__(self, win_parent)
 		self.ui = Ui_MainWindow()
-                self.ui.setupUi(self)
+		self.ui.setupUi(self)
 		# Setup GUI elements
-                # Populate the available Providers
-                for value in list_providers:
+		# Populate the contact list
+		self.comboBox_contact_populate()
+		# Populate the available Providers
+		for value in list_providers:
 			self.ui.comboBox_SetupProvider.addItem(value)
 		CT = time.localtime(time.time())
 		# Set the year for scheduling SMS. From now to the next three years
 		YEAR = CT[0]
 		for YEARS in range(YEAR,YEAR + 3, 1):
-		    	self.ui.comboBox_AdvancedScheduleStartYear.addItem(str(YEARS))
+			self.ui.comboBox_AdvancedScheduleStartYear.addItem(str(YEARS))
 			self.ui.comboBox_AdvancedScheduleStopYear.addItem(str(YEARS))
 		# Try to update the GUI to the last used Account, else skip this
-                try:
+		try:
 			PROVIDER, ALIAS, LOGIN, PASSWORD = c.execute('SELECT provider, alias, login, password FROM login WHERE last=1').fetchall()[0]
 			# Populate all Acounts
 			ACCOUNTS = c.execute('SELECT alias FROM login').fetchall()
@@ -72,21 +81,106 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 			#PASSWORD = base64.b64decode(PASSWORD)
 			self.ui.lineEdit_SetupPassword.setText(PASSWORD)
 		except IndexError:
-			pass
+			provider_index = -1
 
-		# Conect to some buttons and make them execute a method of the class MainWindow
+		# Connect to some buttons and make them execute a method of the class MainWindow
 		QtCore.QObject.connect(self.ui.pushButton_Send, QtCore.SIGNAL("clicked()"), self.on_send_clicked)
+		QtCore.QObject.connect(self.ui.comboBox_contact, QtCore.SIGNAL("activated(QString)"), self.comboBox_contact_itemActivated)
+		QtCore.QObject.connect(self.ui.comboBox_number, QtCore.SIGNAL("activated(QString)"), self.comboBox_number_itemActivated)
 		QtCore.QObject.connect(self.ui.pushButton_SetupSave, QtCore.SIGNAL("clicked()"), self.on_SetupSave_clicked)
 		QtCore.QObject.connect(self.ui.plainTextEdit_SMS, QtCore.SIGNAL("textChanged()"), self.UpdateChars)
 		QtCore.QObject.connect(self.ui.comboBox_SetupAlias, QtCore.SIGNAL("activated(QString)"), self.on_SetupAlias_activated)
 		QtCore.QObject.connect(self.ui.pushButton_SetupDelete, QtCore.SIGNAL("clicked()"), self.on_SetupDelete_clicked)
-		QtCore.QObject.connect(self.ui.pushButton_SetupDelete, QtCore.SIGNAL("clicked()"), self.on_SetupDelete_clicked)
 		QtCore.QObject.connect(self.ui.comboBox_AdvancedScheduleStartMonth, QtCore.SIGNAL("activated(QString)"), self.on_StartDate_changed)
 		QtCore.QObject.connect(self.ui.comboBox_AdvancedScheduleStopMonth, QtCore.SIGNAL("activated(QString)"), self.on_StopDate_changed)
 		QtCore.QObject.connect(self.ui.comboBox_AdvancedScheduleStartYear, QtCore.SIGNAL("activated(QString)"), self.on_StartDate_changed)
 		QtCore.QObject.connect(self.ui.comboBox_AdvancedScheduleStopYear, QtCore.SIGNAL("activated(QString)"), self.on_StopDate_changed)
+		self.enableTabs(provider_index)
 
-	# Class which is called to get the propper number of days for the schedule start date
+	def enableTabs(self, provider_index):
+		# Enable the SMS tab if we have an account, otherwise not
+		ALIAS = unicode(self.ui.comboBox_SetupAlias.currentText())
+		idx = self.ui.tabWidget.indexOf(self.ui.Tab_SMS)
+		if ALIAS:
+			self.ui.tabWidget.setTabEnabled(idx, True)
+		else:
+			self.ui.tabWidget.setTabEnabled(idx, False)
+		# Enable the "advanced" options if supported
+		idx = self.ui.tabWidget.indexOf(self.ui.Tab_Advanced)
+		if (provider_index >= 0) and providers[provider_index].hasAdvancedFeatures():
+			self.ui.tabWidget.setTabEnabled(idx, True)
+		else:
+			self.ui.tabWidget.setTabEnabled(idx, False)
+
+	# Class wich is called to get the contacts from the evolution backend
+	def comboBox_contact_populate(self):
+		name_attributes = [
+			"full-name",
+			"given-name",
+			"family-name",
+			"nickname"]
+
+		phone_attributes = [
+			"assistant-phone",
+			"business-phone",
+			"business-phone-2",
+			"business-fax",
+			"callback-phone",
+			"car-phone",
+			"company-phone",
+			"home-phone",
+			"home-phone-2",
+			"home-fax",
+			"isdn-phone",
+			"mobile-phone",
+			"other-phone",
+			"other-fax",
+			"pager",
+			"primary-phone",
+			"radio",
+			"telex",
+			"tty",]
+
+		single_contact = []
+		single_contact_numbers = []
+		self.contact_list = []
+		self.phone_list = []
+
+		abook = evolution.ebook.open_addressbook("default")
+		contacts = abook.get_all_contacts()
+		#help(contacts)
+		contacts.sort(key=lambda obj: obj.get_property(name_attributes[2]))
+
+		for econtact in contacts:
+			#help(econtact)
+			single_contact = [econtact.get_property(name_attributes[2]), econtact.get_property(name_attributes[1])]
+			for phone_attribute in phone_attributes:
+				single_contact_numbers.append(econtact.get_property(phone_attribute))
+			self.contact_list.append(single_contact)
+			self.phone_list.append(single_contact_numbers)
+			single_contact = []
+			single_contact_numbers = []
+		for contact in self.contact_list:
+			item = "%s, %s" %(contact[0], contact[1])
+			self.ui.comboBox_contact.addItem(item)
+
+	def comboBox_contact_itemActivated(self):
+		self.ui.comboBox_number.clear()
+		contact_index = self.ui.comboBox_contact.currentIndex()
+		print contact_index
+		print self.phone_list[contact_index]
+		for item in self.phone_list[contact_index]:
+			try:
+				self.ui.comboBox_number.addItem(item)
+			except:
+				pass
+		self.ui.lineEdit_Number.setText(self.phone_list[contact_index][11])
+
+	def comboBox_number_itemActivated(self):
+		item = self.ui.comboBox_number.currentText()
+		self.ui.lineEdit_Number.setText(item)
+
+	# Method which is called to get the proper number of days for the schedule start date
 	def on_StartDate_changed(self):
 		cal = calendar.Calendar()
 		sYEAR = int(self.ui.comboBox_AdvancedScheduleStartYear.currentText())
@@ -97,7 +191,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 				DAY = '%02d' %(DAY)
 				self.ui.comboBox_AdvancedScheduleStartDay.addItem(str(DAY))
 
-	# Class which is called to get the propper number of days for the schedule stop date
+	# Method which is called to get the proper number of days for the schedule stop date
 	def on_StopDate_changed(self):
 		cal = calendar.Calendar()
 		sYEAR = int(self.ui.comboBox_AdvancedScheduleStopYear.currentText())
@@ -108,7 +202,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 				DAY = '%02d' %(DAY)
 				self.ui.comboBox_AdvancedScheduleStopDay.addItem(str(DAY))
 
-	# Class which is called to get the number of characters typed in plainTextEdit_SMS and update lcdNumber_SMSChars
+	# Method which is called to get the number of characters typed in plainTextEdit_SMS and update lcdNumber_SMSChars
 	def UpdateChars(self):
 		SMS_CHARS = self.ui.plainTextEdit_SMS.document().characterCount() - 1
 		self.ui.label_SMSChars.setText(str(SMS_CHARS))
@@ -116,7 +210,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		SMS_COUNT = int(math.ceil(SMS_CHARS / 160) + 1)
 		self.ui.label_SMSCount.setText(str(SMS_COUNT))
 
-	# Class which ubdates the GUI to the Account selected in comboBox_SetupAlias
+	# Method which updates the GUI to the Account selected in comboBox_SetupAlias
 	def on_SetupAlias_activated(self):
 		# Get the selected Account
 		ALIAS = unicode(self.ui.comboBox_SetupAlias.currentText())
@@ -131,6 +225,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		self.ui.lineEdit_SetupNewAlias.setText(ALIAS)
 		self.ui.lineEdit_SetupLogin.setText(LOGIN)
 		self.ui.lineEdit_SetupPassword.setText(PASSWORD)
+		self.enableTabs(provider_index)
 		# Remove the last account flag to set it on the new/updated profile
 		c.execute('UPDATE login SET last=0 WHERE last=1')
 		# Set the last flag on the new Account
@@ -141,7 +236,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 	def on_SetupDelete_clicked(self):
 		# Get the current selected Account
 		ALIAS = unicode(self.ui.comboBox_SetupAlias.currentText())
-		# Delet it form the DB
+		# Delete it from the DB
 		c.execute('DELETE FROM login WHERE alias=?', (ALIAS,))
 		conn.commit()
 		# Remove the Account from comboBox_SetupAlias
@@ -154,11 +249,12 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 			self.ui.lineEdit_SetupNewAlias.clear()
 			self.ui.lineEdit_SetupLogin.clear()
 			self.ui.lineEdit_SetupPassword.clear()
+			self.enableTabs(-1)
 		else:
 			self.on_SetupAlias_activated()
 		QtGui.QMessageBox.information(self, 'www2sms', "Account %s deleted" %(ALIAS))
 
-	# Class which is called, when an account is created/updated
+	# Method which is called, when an account is created/updated
 	def on_SetupSave_clicked(self):
 		# Gather Account information
 		PROVIDER = unicode(self.ui.comboBox_SetupProvider.currentText())
@@ -185,8 +281,9 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		self.ui.comboBox_SetupProvider.setCurrentIndex(provider_index)
 		self.ui.lineEdit_SetupLogin.setText(LOGIN)
 		self.ui.lineEdit_SetupPassword.setText(PASSWORD)
+		self.enableTabs(provider_index)
 
-	# Class which is called when we want to send the sms. It actually calls a method on the providers class
+	# Method which is called when we want to send the sms. It actually calls a method on the providers class
 	def on_send_clicked(self):
 		self.ui.pushButton_Send.setText("Sending...")
 		# Really ready to send?
@@ -196,9 +293,9 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		    	# Read Scheduling settings if activated
 
 			SCHEDULE = []
-			if self.ui.radioButton_AdvancedScheduleRepeat.isChecked():
+			if self.ui.checkBox_AdvancedScheduleRepeat.isChecked():
 				# SCHEDULE Layout: ([RECURRENCY, STOPTrueFalse], [YEARstart, YEARstop], [MONTHstart, MONTHstop], [DAYstart, DAYstop], [HOUREstart, HOUREstop], [MINUTEstart, MINUTEstop])
-				SCHEDULE.append([ str(self.ui.comboBox_AdvancedRecurrence.currentText()), self.ui.radioButton_AdvancedScheduleStop.isChecked() ])
+				SCHEDULE.append([ str(self.ui.comboBox_AdvancedRecurrence.currentText()), self.ui.checkBox_AdvancedScheduleStop.isChecked() ])
 				SCHEDULE.append([ str(self.ui.comboBox_AdvancedScheduleStartYear.currentText()), str(self.ui.comboBox_AdvancedScheduleStopYear.currentText()) ])
 				SCHEDULE.append([ str(self.ui.comboBox_AdvancedScheduleStartMonth.currentText()), str(self.ui.comboBox_AdvancedScheduleStopMonth.currentText()) ])
 				SCHEDULE.append([ str(self.ui.comboBox_AdvancedScheduleStartDay.currentText()), str(self.ui.comboBox_AdvancedScheduleStopDay.currentText()) ])
@@ -216,7 +313,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 				SENDER = self.ui.lineEdit_AdvancedText.text()
 
 			# Read Special
-			FLASH = self.ui.radioButton_AdvancedFlashSMS.isChecked()
+			FLASH = self.ui.checkBox_AdvancedFlashSMS.isChecked()
 
 			# Read the login
 			LOGIN = unicode(self.ui.lineEdit_SetupLogin.text())
@@ -228,23 +325,24 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 			SMSTEXT = unicode(self.ui.plainTextEdit_SMS.toPlainText())
 			# Read the provider to use
 			PROVIDER = unicode(self.ui.comboBox_SetupProvider.currentText())
-			# Instanziate the providers class
-			send_SMS = providers()
-			# Call the method PROVIDER from the providers class and pass the number and text to send.
-			STATUS, MSG = getattr(send_SMS, PROVIDER)(NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE, SENDER, FLASH)
+			# Get the provder
+			provider_index = self.ui.comboBox_SetupProvider.findText(PROVIDER)
+			provider = providers[provider_index]
+			# send the SMS
+			STATUS, MSG = provider.sendSMS(NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE, SENDER, FLASH)
 			if STATUS:
 				self.on_send_success(MSG)
 			else:
 				self.on_send_failed(MSG)
 		self.ui.pushButton_Send.setText("Send")
 
-	# Class wich is called from providers class when the sms was successfully send
+	# Method which is called from providers class when the sms was successfully send
 	def on_send_success(self,MSG):
 		QtGui.QMessageBox.information(self, 'www2sms', MSG)
 		self.ui.lineEdit_Number.clear()
 		self.ui.plainTextEdit_SMS.clear()
 
-	# Class wich is called from providers class when the sms failed to send
+	# Method which is called from providers class when the sms failed to send
 	def on_send_failed(self,MSG):
 		QtGui.QMessageBox.warning(self, 'www2sms', MSG)
 
@@ -257,7 +355,7 @@ def mk_db():
 	    pass
 	else:
 	    raise Exception("Failed to create %s. (%s, %s)" %(CONFIG_DIR, errno, strerror))
-	# Chanke to the config dir.
+	# Change to the config dir.
 	os.chdir(CONFIG_DIR)
 	conn = sqlite3.connect('www2sms.db')
 	c = conn.cursor()
