@@ -4,8 +4,26 @@
 import sys, re, mechanize, cookielib, base64, urllib2
 
 # Class which holds the provider methods.
-class providers:
-	def o2online(self, NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE, SENDER, FLASH):
+class provider:
+	# get the printable name for this provider
+	def getName(self):
+		raise Exception('provider class is a base class only')
+	# eventually we will have to separate the advanced features out, and return these as
+	# canSchedule(), canSetSender(), canFlash(), but with the current providers these
+	# are all either on or off, so we use a single method to describe them
+	def hasAdvancedFeatures(self):
+		return False
+	# send the sms
+	def sendSMS(self, NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE=None, SENDER=None, FLASH=None):
+		return False, "SMS sending not implemented"
+
+# o2online.de 1.0
+class provider_o2online_de(provider):
+	def getName(self):
+		return 'o2online (Deutschland)'
+	def hasAdvancedFeatures(self):
+		return True
+	def sendSMS(self, NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE=None, SENDER=None, FLASH=None):
 		LOGINURL = 'http://www.o2online.de/'
 		execute = True
 		if execute:
@@ -200,4 +218,79 @@ class providers:
 			#<SelectControl(EndDateYear=[*2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015])>
 			#<SelectControl(EndDateHour=[*00, 01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23])>
 			#<SelectControl(EndDateMin=[*00, 15, 30, 45])>>
-			#---------------------------
\ No newline at end of file
+			#---------------------------
+
+# vodafone.nl 1.0
+class provider_vodafone_nl(provider):
+	def getName(self):
+		return 'Vodefone Nederland'
+	def hasAdvancedFeatures(self):
+		return False
+	def sendSMS(self, NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE=None, SENDER=None, FLASH=None):
+		# TODO: synchronize number verification with main code (UpdateChars() perhaps?)
+		# TODO: block SMS longer than 1 message
+		# TODO: enforce maximum SMS size of 125 characters (*not* 160)
+		# TODO: check whether sending worked
+		# TODO: handle "2 per day maximum"
+		# TODO: localization support (at least English and Dutch)
+
+		# get rid of whitespace and dots
+		number = re.sub(r'(\s|\.)+', '', NUMBER)
+
+		# handle a couple of number formats (only mobile numbers allowed)
+		if re.search(r'^\+316\d{8}$', number):	# ITU format, +31 is the Netherlands
+			number = number[3:]
+		elif re.search(r'^06\d{8}$', number):	# local dialing format
+			number = number[1:]
+		else:
+			return False, 'Only Dutch mobile phone numbers allowed'
+
+		# set up our browser
+		br = mechanize.Browser()
+		cj = cookielib.LWPCookieJar()
+		br.set_cookiejar(cj)
+
+		# login
+		try:
+			br.open('https://my.vodafone.nl/prive/my_vodafone')
+			br.select_form(name='login')
+			br.form['username'] = LOGIN
+			br.form['password'] = PASSWORD
+			br.submit()
+		except:
+			return False, "Error logging in"
+
+
+		# verify our login worked and return error if not
+		br.response().read()
+		try:
+			br.find_link(text='Uitloggen')
+		except mechanize.LinkNotFoundError:
+			return False, 'Login failed'
+
+		# send the SMS
+		try:
+			br.open('https://my.vodafone.nl/Prive/My_Vodafone/gratis_sms_versturen')
+		except:
+			return False, 'Error opening free SMS web page'
+		response = br.response().read()
+		if re.search(r'Je hebt het maximum aantal gratis SMS\'jes bereikt voor vandaag.', response):
+			return False, "Provider responded: Je hebt het maximum aantal gratis SMS'jes bereikt voor vandaag."
+
+		try:
+			br.select_form(name='controle')
+			br.form['phoneNumber'] = number
+			br.form['body'] = SMSTEXT
+			br.submit()
+		except:
+			print response
+			return False, 'Error submitting SMS form'
+
+		# check to see if our send worked
+		response = br.response().read()
+		if re.search(r'Je sms\'je is verstuurd.', response):
+			return True, "Provider responded: Je sms'je is verstuurd."
+		elif re.search(r'Je hebt het maximum aantal gratis SMS\'jes voor vandaag verstuurd.', response):
+			return False, "Provider responded: Je hebt het maximum aantal gratis SMS'jes voor vandaag verstuurd."
+		else:
+			return False, 'Message not sent'
